#!/usr/bin/env /proj/axaf/bin/perl

# this may be run either directly as a script or use'd as a module

use strict;
use warnings;

package SysPathSubst;

use autouse Cwd => qw( realpath );
use autouse 'File::Spec::Functions' => qw(catdir);

my $VERSION = '2.1.18';

my $prog;


run() unless caller();

use vars qw( @ISA @EXPORT_OK );

require Exporter;
@ISA = qw( Exporter );
@EXPORT_OK = qw( syspathsubst compact_paths );

sub prog
{
    require File::Basename;
    $prog ||= File::Basename::basename( $0 );
}

sub run
{
    my $ok = eval { main() };

    if ( $@ )
    {
	my $error = $@;

	my $prog = prog();
	print STDERR "$prog: $_\n" foreach split /\n/, $error;
	exit 1;
    }

    exit ! $ok;
}


sub main
{
    my %opt = opts();

    help(0)   if $opt{help};
    help(2)   if $opt{usage};
    version() if $opt{version};


    my $paths = syspathsubst( $opt{delimiter},
			      $opt{components},
			      @{$opt{templates}} );

    print $paths, "\n" if $paths;
}

sub syspathsubst
{
    my ( $delimiter, $components, @templates ) = @_;

    return compact_paths( $delimiter,
			  map { pathsubst( $components, $_ ) } @templates
			);
}

sub compact_paths
{
    my $delimiter = shift;

    my %in;

    my @paths = 
      grep { $_ && ! $in{$_}++ } 
      map { 'ARRAY' eq ref($_) ? catdir( @$_ ) : $_ } @_;

    return wantarray() ? @paths : join( $delimiter, @paths );
}


sub pathsubst
{
    my ( $components, $tpl ) = @_;

    $tpl = catdir( @$tpl )
      if 'ARRAY' eq ref $tpl;

    my $resolvelinks = $tpl =~ /%[lL]/;
    my $firstvalid   = $tpl =~ /%[LP]/;
    $tpl =~ s/%[pl]/%s/i;

    my @paths;

    # do minimum if nothing to fill
    if ( $tpl !~ /%s/ )
    {
	push @paths, $tpl
	  if -e $tpl
    }

    # templates to fill
    else
    {
	for my $cmp ( @$components )
	{
	    (my $tpath = $tpl) =~ s/%s/$cmp/;

	    $tpath = realpath( $tpath ) || $tpath
	      if $resolvelinks;

	    next unless -e $tpath;
	    push @paths, $tpath;
	    last if $firstvalid;
	}
    }
    return @paths;
}


sub opts
{
    require Getopt::Long;
    my %opt = ( delimiter => ' ' );

    eval {
        local $SIG{ __WARN__ } = sub { die $_[0] };

	Getopt::Long::GetOptions ( \%opt,
		     qw/
			components=s
			delimiter=s
			help
			usage
			version
			/,
		   );
    };

    die $@ if $@;

    return %opt if grep { $opt{$_} } qw/ help usage version /;

    die "missing path templates\n" unless @ARGV;

    $opt{templates} = [ map { split(':', $_ ) } @ARGV ];

    if ( ! defined $opt{components} )
    {
	chomp ($opt{components} = qx( /proj/axaf/simul/bin/sysarch platforms ));
	die 'error running sysarch' if $?;
    }

    $opt{components} = [ split(':', $opt{components} ) ];

    $opt{delimiter} = "\n" if $opt{delimiter} eq '\n';

    return %opt;
}


sub version
{
    print prog(), " ", $VERSION, "\n";
    exit 0;
}

sub help
{
  my ( $verbose ) = @_;

  require Pod::Usage;
  Pod::Usage::pod2usage ( { -exitval => 0, -verbose => $verbose } );
}

1;

__END__

=pod

=head1 NAME

SysPathSubst.pm - resolve system specific paths

=head1 SYNOPSIS

  use SysPathSubst qw( syspathsubst compact_path);
  $path = syspathsubst( $delimiter, \@components, \@templates );
  $path = compact_paths( $delimiter, @paths );


=head1 DESCRIPTION

This package will substitute components into each element in a list
of path templates.

Typically B<SysPathSubst> is used on networks with multiple operating
system and hardware combinations where execution and library paths
differ only by a system specific path component.  Using this package
allows one to more easily specify those paths.

=head1 FUNCTIONS

The B<SysPathSubst> module has two functions available for export:

=over

=item syspathsubst

  $path = syspathsubst( $delimiter, \@components, \@templates );
  @paths = syspathsubst( undef, \@components, \@templates );

Perform substitution in the passed paths, and return the unique
subset.  Uniqueness is determined by a simple string comparison, not
by resolving the paths, so that if two paths resolve to the same disk
location but one uses symbolic links, they will appear distinct.  To
avoid this, use a path template token which ensures that links are
resolved.

C<@components> is an array of components to be
substituted into the paths, in the order they should be checked.
C<@templates> is an array of templates (see L</Path Templates>).

If an element in C<@templates> is itself an array, the path create by
passing to B<File::Spec::catdir> will be used in its place.

If called in a scalar environment, the subset is returned as a string
of paths, with the paths separated by C<$delimiter>, otherwise it is
returned as a list (and C<$delimiter> is ignored).

=item compact_paths

  $path = compact_paths( $delimiter, @paths );
  @paths = compact_paths( undef, @paths );

Return the unique subset of the passed paths.  Uniqueness is
determined by a simple string comparison, not by resolving the paths.
If called in a scalar environment, the subset is returned as a string
of paths, with the paths separated by C<$delimiter>, otherwise it is
returned as a list (and C<$delimiter> is ignored).


If an element in C<@paths> is itself an array, the path created by
passing to B<File::Spec::catdir> will be used in its place.

=back


=head2 Path Templates


A path template is a string which can contain zero or one of a set of
special tokens.  The tokens indicate where in a path the system
specific component will be substituted, as well as the value of two
attributes, B<resolvelinks> and B<firstvalid>:

  Token   resolvelinks  firstvalid
  -----   -----------  ----------
   %s        false       false
   %p        false       false
   %P        false       true
   %l        true        false
   %L        true        true

If B<resolvelinks> is true, then after substitution any symbolic links
in the path are fully resolved.  This always results in an absolute
path.

If B<firstvalid> is true, then only the first existing path resulting
from the in order substitution of the system specific components is
retained.

For example, if the list of system specific components is

  c d e f

and the existent paths are

  /a/c /a/e /a/f

Then the path template C</a/%p> would result in a list of paths

  /a/c:/a/e:/a/f

while the path template C</a/%P> would result in

  /a/c

=head1 COPYRIGHT AND LICENSE

Copyright 2007 The Smithsonian Astrophysical Observatory

This software is released under the GNU General Public License.  You
may find a copy at: L<http://www.fsf.org/copyleft/gpl.html>

=head1 VERSION

This documents version 2.1.18.

=head1 Author

Diab Jerius E<lt>djerius@cfa.harvard.eduE<gt>
